// Clase de gestión del driver
public class Driver
{
	// Declaración de estados
	private Estado estadoActual;
	private EstInicio estInicio;
	private EstComprobando estComprobando;
	private EstLeyendo estLeyendo;
	private EstAlmacenando estAlmacenando;
	private EstDetenido estDetenido;
	private EstFinal estFinal;
	
	// Clase abstracta Estado	
	abstract class Estado
	{	
		public String nombreEstado;
		
		// Posibles eventos
		public void iniciar()
		{	
			checkError("EstInicio");
		}
		
		public void pasanDosSegundos()
		{
			checkError("EstComprobando");
		}
		
		public void datoRecibido()
		{
			checkError("EstComprobando");
		}
		
		public void errorLectura()
		{
			checkError("EstLeyendo");
		}
		
		public void lecturaOK()
		{
			checkError("EstLeyendo");
		}
		
		public void bufferLleno()
		{
			checkError("EstAlmacenando");
		}
		public void quedaEspacioBuffer()
		{
			checkError("EstAlmacenando");
		}

		public void pasanDiezSegundos()
		{
			checkError("EstDetenido");
		}
		
		public void imprimeEstado()
		{
			System.out.println(nombreEstado);
		}
		
		// Comprueba si en estado correcto
		public void checkError(String supuesto)
		{
			if (!nombreEstado.equals(supuesto))
				System.out.println("Error estado incorrecto");
		}
	}
	
	// Implementa los estados concretos	
	class EstInicio extends Estado
	{
		public EstInicio()
		{
			nombreEstado = "EstInicio";
		}
		
		public void iniciar()
		{
			System.out.println("iniciar()");
			estadoActual = estComprobando;
		}
	}
	// Comprobando registro de estado
	class EstComprobando extends Estado
	{
		public EstComprobando()
		{
			nombreEstado = "EstComprobando";
		}
		
		public void pasanDosSegundos()
		{
			System.out.println("pasanDosSegundos()");
			estadoActual = estComprobando;
		}
		
		public void datoRecibido()
		{
			System.out.println("datoRecibido()");
			estadoActual = estLeyendo;
		}
	}
	
	// Lectura dato del puerto
	class EstLeyendo extends Estado
	{	
		public EstLeyendo()
		{
			nombreEstado = "EstLeyendo";
		}
		
		public void errorLectura()
		{
			System.out.println("ErrorLectura()");
			estadoActual = estDetenido;
		}
		
		public void lecturaOK()
		{
			System.out.println("lecturaOK()");
			estadoActual = estAlmacenando;
		
		}
	}	
	
	// Almacenamiento buffer
	class EstAlmacenando extends Estado
	{
		public EstAlmacenando()
		{
			nombreEstado = "EstAlmacenando";
		}
		
		public void bufferLleno()
		{
			System.out.println("BufferLleno()");
			estadoActual = estDetenido;
		
		}
		public void quedaEspacioBuffer()
		{
			System.out.println("quedaEspacioBuffer()");
			estadoActual = estFinal;
		
		}
	}
	
	// Detenido
	class EstDetenido extends Estado
	{
		public EstDetenido()
		{
			nombreEstado = "EstDetenido";
		}
		
		public void pasanDiezSegundos()
		{
			System.out.println("pasanDiezSegundos()");
			estadoActual = estFinal;
		}
	}
	
	class EstFinal extends Estado
	{
		public EstFinal()
		{
			nombreEstado = "EstFinal";
		}
	
	}
	
	public Driver()
	{	
		estInicio = new EstInicio();
		estComprobando = new EstComprobando();
		estLeyendo = new EstLeyendo();
		estAlmacenando = new EstAlmacenando();
		estDetenido = new EstDetenido();
		estFinal = new EstFinal();
		
		// Estado inicial
		estadoActual = estInicio;	
	}
	
	// Eventos que ocurren en el driver
	public void iniciar()
	{	
		estadoActual.iniciar();
	}
		
	public void pasanDosSegundos()
	{
		estadoActual.pasanDosSegundos();
	}
		
	public void datoRecibido()
	{
		estadoActual.datoRecibido();
		//inhibeInterrupciones();
	}
		
	public void errorLectura()
	{
		estadoActual.errorLectura();
		//activaInterrupciones();
	}
		
	public void lecturaOK()
	{
		estadoActual.lecturaOK();
	}
		
	public void bufferLleno()
	{
		estadoActual.bufferLleno();
		//activaInterrupciones();
	}
	
	public void quedaEspacioBuffer()
	{
		estadoActual.quedaEspacioBuffer();
		//activaInterrupciones();
	}

	public void pasanDiezSegundos()
	{
		estadoActual.pasanDiezSegundos();
	}
	
	public void imprimeEstado()
	{
		estadoActual.imprimeEstado();
	}
}

// Clase principal
class Inicio
{
	public static void main (String[] args) throws java.lang.Exception
	{
		Driver driver = new Driver();
		
		driver.imprimeEstado();
		driver.iniciar();
		driver.imprimeEstado();
		
		driver.imprimeEstado();
		driver.pasanDosSegundos();
		driver.imprimeEstado();
		
		driver.imprimeEstado();
		driver.pasanDosSegundos();
		driver.imprimeEstado();
		
		driver.imprimeEstado();
		driver.datoRecibido();
		driver.imprimeEstado();
		
		driver.imprimeEstado();
		driver.lecturaOK();
		driver.imprimeEstado();
		
		driver.imprimeEstado();
		driver.quedaEspacioBuffer();
		driver.imprimeEstado();
		
		/*driver.imprimeEstado();
		driver.errorLectura();
		driver.imprimeEstado();
		
		driver.imprimeEstado();
		driver.pasanDiezSegundos();
		driver.imprimeEstado();*/
		
		
	}
}